### R code from vignette source 'todorov-PCA-ICORS-2021.Rnw'

###################################################
### code chunk number 1: plot-animals
###################################################
library(rrcov)
data(Animals2)
mcd <- CovMcd(log(Animals2))
plot(log(Animals2), pch=20, cex=2, xlab="log(body)", ylab="log(brain)")
points(log(Animals2)[mcd@best,], col="red", pch=20, cex=2)
title("Optimal (MCD) subset")



###################################################
### code chunk number 2: plot-pca-example
###################################################
## Generate some bivariate random data
library(mvtnorm)
set.seed(12346)
sigma <- matrix(c(4,2,2,3), ncol=2)
x <- rmvnorm(n=500, mean=c(1,2), sigma=sigma)
colMeans(x)
var(x)

## Check package car
##library(car)
##plot(x, xlab = 'x', ylab = 'y')
##ellipse(center = colMeans(x), shape = cov(x), radius = sqrt(qchisq(.975, df=2)), col = 'darkgreen')

## Use package ellipse - for the simple function ellipse()
library(ellipse)

##  Set the confidence level, calculate center and covariance
crit <- 0.975
dist <- sqrt(qchisq(crit, 2))
m <- colMeans(x)
c <- cov(x)

##  Draw the ellipse
##  Set the minimum and maximum of the plot window to the minimum and
##  maximum of the data and the ellipse
e <- ellipse::ellipse(c, centre = m, level=crit)
plot(x, xlab = 'X1', ylab = 'X2', xlim=c(min(c(x[,1], e[,1])), max(c(x[,2], e[,2]))), ylim=c(min(c(x[,2], e[,2])), max(c(x[,2], e[,2]))), pch=21, col="brown", bg="yellow")
lines(e, col="turquoise4")

## Set the center
points(m[1],m[2], pch="+", cex=3, col="coral4")

## Find and draw the diagonals of the ellipse
eA <- eigen(c)
ev <- eA$values
evv <- -1*eA$vectors
p1 <- m + evv[,1] * sqrt(ev[1]) * dist
p2 <- m + evv[,2] * sqrt(ev[2]) * dist

##  lines(rbind(m, p1), col="red", lwd=2)
##  lines(rbind(m, p2), col="green", lwd=2)

arrows(m[1], m[2], x1=p1[1], y1=p1[2], col="red", lwd=2)
arrows(m[1], m[2], x1=p2[1], y1=p2[2], col="blue", lwd=2)
text(7, 6, expression(lambda[1]*p[1]), col="red", cex=1.5)
text(-2, 5, expression(lambda[2]*p[2]), col="blue", cex=1.5)


###################################################
### code chunk number 3: plot-pca-biplot
###################################################
pc <- PcaClassic(iris[, 1:4])
biplot(pc, scale=0, col=list(as.integer(iris$Species)+2, "red"))


###################################################
### code chunk number 4: plot-cpca-hbk
###################################################
pca.scoreplot(PcaClassic(hbk))


###################################################
### code chunk number 5: plot-rpca-hbk
###################################################
pca.scoreplot(PcaCov(hbk))


###################################################
### code chunk number 6: plot-pca-map
###################################################
library(rrcov)
data(hbk)
pc <- PcaClassic(hbk, k=2)
plot(pc, id.n.sd=4, off=0.05, pch=16, col=c(rep("red", 14), rep("blue", 76)))
## The 14 observations shown in red are known to be outliers, however the classical PCA
##  identifies only four of them


###################################################
### code chunk number 7: plot-pca-score
###################################################
library(rrcov)
data(hbk)
pc <- PcaClassic(hbk)       # k=4 (default)
plot(pc, id.n.sd=4, off=0.05, pch=16, col=c(rep("red", 14), rep("blue", 76)))
## The 14 observations shown in red are known to be outliers, however the classical PCA
##  identifies only four of them


###################################################
### code chunk number 8: plot-pca-object
###################################################
library(rrcov)
data(hbk)
pc <- PcaClassic(hbk, 2)
head(pc$od)     # orthogonal distances
head(pc$sd)     # score distances
head(pc$flag)   # outlier identification
pc$cutoff.od    # OD cutoff
pc$cutoff.sd    # SD cutoff


###################################################
### code chunk number 9: plot-pca-animals
###################################################
data(Animals2)

## Classical covariance matrix
cc <- CovClassic(log(Animals2))
xv <- eigen(cc$cov)$values
xvv <- eigen(cc$cov)$vectors

## MCD covariance matrix
mcd <- CovMcd(log(Animals2))
ev <- eigen(mcd$cov)$values
evv <- eigen(mcd$cov)$vectors

plot(mcd, which="tolEllipse", id.n=0, pch=21, bg="blue")
abline(a=2, b=evv[2,1]/evv[1,1], col="red", lwd=2)
abline(a=2, b=xvv[2,1]/xvv[1,1], col="blue", lty="dotted", lwd=2)


###################################################
### code chunk number 10: mcd
###################################################
library(rrcov)
data(hbk)
##  ?hbk


###################################################
### code chunk number 11: plot-pca-hbk-screeplot
###################################################
screeplot(PcaClassic(hbk), main="HBK: Classical PCA")


###################################################
### code chunk number 12: plot-rpc-hbk-screeplot
###################################################
screeplot(PcaCov(hbk), main="HBK: MCD")


###################################################
### code chunk number 13: plot-pca-hbk-outlier1
###################################################
plot(PcaClassic(hbk))


###################################################
### code chunk number 14: plot-rpc-hbk-outlier1
###################################################
plot(PcaCov(hbk))


###################################################
### code chunk number 15: plot-pca-hbk-scoreplot
###################################################
pca.scoreplot(PcaClassic(hbk))


###################################################
### code chunk number 16: plot-rpc-hbk-scoreplot
###################################################
pca.scoreplot(PcaCov(hbk))


###################################################
### code chunk number 17: plot-pca-hbk-outlier2
###################################################
plot(PcaClassic(hbk, k=2))


###################################################
### code chunk number 18: plot-rpc-hbk-outlier2
###################################################
plot(PcaCov(hbk, k=2))


###################################################
### code chunk number 19: plot-plugin-pca-summary
###################################################
library(rrcov)
data(hbk)
pca <- PcaClassic(hbk)      # Classical PCA
rpc1 <- PcaCov(hbk)         # Uses MCD with alpha=0.5 (default) 
rpc2 <- PcaCov(hbk, cov.control=CovControlMMest())
screeplot(rpc1)
pca.scoreplot(rpc1)
plot(rpc1)                   # outlier map
getLoadings(rpc1)
getEigenvalues(rpc1)
getScores(rpc1)       


###################################################
### code chunk number 20: plot-pca-grid
###################################################
    data(bus)
    bus <- as.matrix(bus)

    ## calculate MADN for each variable
    xmad <- apply(bus, 2, mad)
    cat("\nMin, Max of MADN: ", min(xmad), max(xmad), "\n")


    ## MADN vary between 0 (for variable 9) and 34. Therefore exclude
    ##  variable 9 and divide the remaining variables by their MADNs.
    bus1 <- bus[, -9]
    madbus <- apply(bus1, 2, mad)
    bus2 <- sweep(bus1, 2, madbus, "/", check.margin = FALSE)

    rpc  <- PcaGrid(bus2, method="qn")
    ev <- getEigenvalues(rpc)
    cumsum(ev)/sum(ev)


###################################################
### code chunk number 21: plot-pca-bus2
###################################################
    pc  <- PcaClassic(bus2,  k=5)
    plot(pc)


###################################################
### code chunk number 22: plot-pcapp-bus2
###################################################
    rpc  <- PcaGrid(bus2, method="qn", k=5)
    plot(rpc)


###################################################
### code chunk number 23: plot-pca-bus2-loadings
###################################################
    matplot(pc$loadings, ylab="Loadings", main="Classical PCA")


###################################################
### code chunk number 24: plot-pcapp-bus2-loadings
###################################################
    matplot(rpc$loadings, ylab="Loadings", main="Projection-based PCA")


###################################################
### code chunk number 25: plot-sphpca-bus2
###################################################
    sphpca <- PcaLocantore(bus2, k=5)
    plot(sphpca, main="Spherical PCA")


###################################################
### code chunk number 26: plot-pca-bus-robpca
###################################################
    pcaHubert <- PcaHubert(bus2, k=5, mcd=FALSE, alpha=0.5)
    plot(pcaHubert, id.n.od=5, off=0.04, main="ROBPCA")


###################################################
### code chunk number 27: todorov-PCA-ICORS-2021.Rnw:1191-1207
###################################################
library(rospca)
data(Glass)

##  We define a function to calculate the univariate MCD using the
##  internal rrcov function unimcd() and apply it to the data set
##  using the robustbase (undocumented) function doScale(). We set
##  the scale parameter to NULL - in order to do no scaling note
##  that in base::scale() we have scale=FALSE for this purpose.

umcd_center <- function(y)
    rrcov:::unimcd(y, quan=h.alpha.n(0.5, length(y), 1))$tmcd

X <- doScale(Glass, center=umcd_center, scale=NULL)$x
X <- X[, 1:500]
(n <- nrow(X))
(p <- ncol(X))


###################################################
### code chunk number 28: plot-pca-glass-all
###################################################
matplot(t(X), type="l", lty=1)


###################################################
### code chunk number 29: plot-pca-glass-class-ddplot
###################################################
pc <- PcaClassic(X, k=3, scale=FALSE)
plot(pc, id.n.sd=0, id.n.od=0)


###################################################
### code chunk number 30: plot-pca-glass-class-scoreplot
###################################################
pca.scoreplot(pc)


###################################################
### code chunk number 31: plot-pca-glass-class-loadingsplot
###################################################
matplot(pc$loadings, type="l", lty=1, lwd=2, ylab="",
    col=c(2,3,4), main="Classical PCA")
text(x=c(140, 300, 380), y=c(0.25, 0.2, 0.25), label=c("1", "2", "3"))


###################################################
### code chunk number 32: todorov-PCA-ICORS-2021.Rnw:1279-1280
###################################################
rpc <- PcaHubert(X, k=3, scale=FALSE, alpha=0.7)


###################################################
### code chunk number 33: plot-pca-glass-robpca-loadingsplot
###################################################
matplot(rpc$loadings, type="l", lty=1, lwd=2, ylab="", col=c(2,3,4), main="ROBPCA")
text(x=c(140, 300, 380), y=c(0.25, 0.2, 0.25), label=c(3,1,2))


###################################################
### code chunk number 34: plot-pca-glass-robpca-ddplot
###################################################
plot(rpc, id.n.sd=0, id.n.od=0)
text(x=7.5, y=1100, label="143-179")
text(x=5.1, y=1350, label="57-63,")
text(x=5.1, y=1300, label="74-76")
text(x=1.8, y=1350, label="30")
text(x=1, y=1150, label="22,23")
text(x=5, y=1050, label="180")


###################################################
### code chunk number 35: plot-pca-glass-good
###################################################
xa <- c(57:63, 74:76)               # bad leverage points 57-63 and 74-76; grp==4
xb <- 143:179                       # bad leverage points
xc <- c(22, 23, 30)                 # orthogonal outliers 22, 23, 30
xd <- 180
xgood <- 1:nrow(X)
xgood <- xgood[which(!(xgood %in% xa))]
xgood <- xgood[which(!(xgood %in% xb))]
xgood <- xgood[which(!(xgood %in% xc))]
xgood <- xgood[which(!(xgood %in% xd))]
matplot(t(X[xgood,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")


###################################################
### code chunk number 36: plot-pca-glass-bad1
###################################################
matplot(t(X[xb,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")


###################################################
### code chunk number 37: plot-pca-glass-bad2
###################################################
matplot(t(X[xa,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")


###################################################
### code chunk number 38: plot-pca-glass-orthogonal
###################################################
matplot(t(X[xc,]), type="l", lty=1, ylim=c(min(X), max(X)), ylab="")


###################################################
### code chunk number 39: todorov-PCA-ICORS-2021.Rnw:1412-1417
###################################################
## install.packages("remotes")
##  remotes::install_github("valentint/rrcov")
data(fruit)
dim(fruit)
table(fruit$cultivar)


###################################################
### code chunk number 40: adjboxplot1
###################################################
### Hubert and Vandervieren (2006), p. 10, Fig. 4.
data(coal, package = "boot")
coaldiff <- diff(coal$date)
hist(coaldiff)


###################################################
### code chunk number 41: adjboxplot2
###################################################
op <- par(mfrow = c(1,2))
boxplot(coaldiff, main = "Original Boxplot")
adjbox(coaldiff, main  = "Adjusted Boxplot")
par(op)


###################################################
### code chunk number 42: machines
###################################################
library(rrcov)
data(machines)
rownames(machines) <- NULL      # names are too long
##  ?machines


###################################################
### code chunk number 43: pca-machines-1
###################################################
rpc1 <- PcaHubert(machines, k=3, scale=TRUE)
plot(rpc1)


###################################################
### code chunk number 44: pca-machines-2
###################################################
set.seed(123456)
rpc3 <- PcaHubert(machines, k=3, scale=TRUE, mcd=FALSE, skew=TRUE, maxdir=1000)
plot(rpc3, main="Robust PCA: skew=TRUE")


###################################################
### code chunk number 45: sparse-pca-1
###################################################
library(pls)
library(rrcovHD)
data(yarn)

plot(1:268, yarn$NIR[1,], type="n",
    xlab="Wavenumber", ylab="Intensity",
    main="Original data")

for(i in 1:28)
    lines(1:268, yarn$NIR[i,], type="l", lwd=1.5, col="blue")


###################################################
### code chunk number 46: sparse-pca-2
###################################################
## First two PCA
col=c("red","green")
yarn.pca <- PcaGrid(yarn$NIR, method="sd")@loadings[,1:2]
ylim <- c(min(yarn.pca), max(yarn.pca))

plot(1:268, yarn.pca[,1], type="n", ylim=ylim,
    xlab="Wavenumber", ylab="PCA loadings",
    main = "PCA")

abline(h=0, lty="dashed", lwd=2)

for(i in 1:2)
    lines(1:268, yarn.pca[,i], type="l", lwd=2, col=col[i])
legend("topright", legend=c("PC 1", "PC 2"), col=col, lty="solid", lwd=2)


###################################################
### code chunk number 47: sparse-pca-3
###################################################
## First two PCA - Sparse
col=c("red","green")
yarn.pca <- SPcaGrid(yarn$NIR, method="sd", lambda=4)@loadings[,1:2]
ylim <- c(min(yarn.pca), max(yarn.pca))
plot(1:268, yarn.pca[,1], type="n", ylim=ylim,
    xlab="Wavenumber", ylab="SPCA loadings",
    main="Sparse PCA")

abline(h=0, lty="dashed", lwd=2)

for(i in 1:2)
    lines(1:268, yarn.pca[,i], type="l", col=col[i], lwd=2)
legend("topright", legend=c("PC 1", "PC 2"), col=col, lty="solid", lwd=2)


###################################################
### code chunk number 48: sparse-pca-4
###################################################
    library(mvtnorm)        # multivariate data with outliers
    set.seed(6543)
    x <- rbind(rmvnorm(200, rep(0, 6), diag(c(5, rep(1,5)))),
                rmvnorm( 15, c(0, rep(20, 5)), diag(rep(1, 6))))

    lambda <- c (0.23, 0.34, 0.005) #  applying sparse PCA
    SPcaGrid (x, k = 3, lambda = lambda)


###################################################
### code chunk number 49: diabetes
###################################################
library(rrcov)
data(diabetes)
##  ?diabetes


###################################################
### code chunk number 50: diabetes1
###################################################
dim(diabetes)
head(diabetes)
table(diabetes$group)


###################################################
### code chunk number 51: diabetes-lda
###################################################
### Classical LDA
lda <- LdaClassic(group~insulin+glucose+sspg, data=diabetes)
(pr <- predict(lda))


###################################################
### code chunk number 52: diabetes-mcd
###################################################
## Robust LDA (using MCD group means and covariance matrix)
rlda <- Linda(group~insulin+glucose+sspg, data=diabetes)
(rpr <- predict(rlda))


###################################################
### code chunk number 53: diabetes-ogk
###################################################
## Robust LDA (using OGK group means and covariance matrix)
rlda.OGK <- Linda(group~insulin+glucose+sspg, data=diabetes, cov.control=CovControlOgk())
(rpr <- predict(rlda.OGK))


###################################################
### code chunk number 54: diabetes-qda-mcd
###################################################
## Robust QDA with MCD (default)
rqda <- QdaCov(group~insulin+glucose+sspg, data=diabetes)
(qpr <- predict(rqda))


###################################################
### code chunk number 55: diabetes-pca
###################################################
## Outlier map with ROBPCA
plot(PcaHubert(diabetes[, -ncol(diabetes)]), id.n.od=10)


